@@ -1,6 +1,6 @@
 import pkgutil
 from pathlib import Path
-from typing import Any, Dict, List, Optional, cast
+from typing import Any, Dict, List, Optional, cast, Union, Tuple
 
 import semver
 import typer
@@ -17,6 +17,9 @@ from alteia_cli.sdk import alteia_sdk
 # PyInquirer is not maintained anymore and uses a not up-to-date version
 # of prompt-toolkit.
 import collections.abc
+
+from alteia_cli.utils import print_error, print_ok, green_bold, blue_bold, print_warn
+
 setattr(collections, 'Iterable', collections.abc.Iterable)
 setattr(collections, 'Mapping', collections.abc.Mapping)
 from PyInquirer import Separator, Token, prompt, style_from_dict  # noqa: E402
@@ -617,7 +620,12 @@ the id of the root company.
 Note that if you specifically shared the analytic with a company
 of your domain, the analytic will still be shared with that company.'''
         )):
-    """ Unshare an analytic """
+    """ Unshare an analytic (DEPRECATED: use unexpose instead) """
+    print_warn(
+        'Deprecation Warning: '
+        'Do not use this command `analytics unshare`. '
+        'Use `analytics unexpose` command instead. '
+    )
     sdk = alteia_sdk()
     analytics = find_analytics(sdk, analytic_name=analytic_name,
                                version_range=version)
@@ -666,7 +674,7 @@ of your domain, the analytic will still be shared with that company.'''
             no_versions_shared_with_company = True
             all_versions_owned_by_company = True
             for version in analytics:
-                if comp['_id'] in version.companies:
+                if comp['_id'] in getattr(version, 'companies', []):
                     no_versions_shared_with_company = False
                 if comp['_id'] != version.company:
                     all_versions_owned_by_company = False
@@ -706,7 +714,7 @@ of your domain, the analytic will still be shared with that company.'''
     for analytic in analytics:
         companies_to_unshare_with = cast(
             List[ResourceId],
-            [c for c in companies if c in analytic.companies]
+            [c for c in companies if c in getattr(analytic, 'companies', [])]
         )
         try:
             for company in companies_to_unshare_with:
@@ -758,7 +766,12 @@ This has the effect to share the analytic with all the
 companies of your domain and is equivalent to using the
 --company option providing the id of the root company.'''
         )):
-    """ Share an analytic """
+    """ Share an analytic (DEPRECATED: use expose instead) """
+    print_warn(
+        'Deprecation Warning: '
+        'Do not use this command `analytics share`. '
+        'Use `analytics expose` command instead. '
+    )
     sdk = alteia_sdk()
 
     permission_name = 'share-analytic-with-company:external'
@@ -800,7 +813,7 @@ companies of your domain and is equivalent to using the
             all_versions_shared_with_company = True
             if all_versions_of_root_shared_with_company is False:
                 for version in analytics:
-                    if comp['_id'] not in version.companies:
+                    if comp['_id'] not in getattr(version, 'companies', []):
                         all_versions_shared_with_company = False
                         break
             if all_versions_shared_with_company:
@@ -849,7 +862,7 @@ companies of your domain and is equivalent to using the
     for analytic in analytics:
         companies_to_share_with = cast(
             List[ResourceId],
-            [c for c in companies if c not in analytic.companies]
+            [c for c in companies if c not in getattr(analytic, 'companies', [])]
         )
         try:
             for company in companies_to_share_with:
@@ -917,3 +930,385 @@ def delete(analytic_name: str = typer.Argument(...),
             .format(analytic_name, analytic.version),
             fg=typer.colors.GREEN
         )
+
+
+def get_domain_root_company_from_current_user(sdk: SDK) -> Tuple[str, ResourceId]:
+    # Assuming a user can only belong to one domain
+    user = sdk.users.describe()
+    first_company_id = user.companies[0]['id']
+    first_company = cast(Resource, sdk.companies.describe(first_company_id))
+    root_company_id = getattr(first_company, 'root_company', None)
+    if root_company_id is None:
+        root_company_id = first_company.id
+
+    # companies always have a domain, which is same as their root company
+    domain_name = first_company.domain
+
+    return domain_name, root_company_id
+
+
+def get_root_companies_of_domains(
+        sdk: SDK,
+        domain_names: Union[List[str], str] = None,
+        raise_if_not_found: bool = True,
+) -> List[Tuple[str, ResourceId]]:
+    if domain_names is None:
+        # no given domains, return the user one
+        return [get_domain_root_company_from_current_user(sdk)]
+
+    if isinstance(domain_names, str):
+        # assume you give many domains separated by comma
+        domain_names = domain_names.split(',')
+
+    filters = {
+        'domain': {'$in': domain_names},
+        'root_company': {'$eq': None},
+    }
+    fields = {'include': ['_id', 'domain']}
+    found_root_companies = cast(
+        List[Resource],
+        sdk.companies.search(filter=filters, fields=fields)
+    )
+    # it must have same number of domains than number of found root companies
+    found_domains = [c.domain for c in found_root_companies]
+    not_found_domains = list(set(domain_names).difference(set(found_domains)))
+
+    if len(not_found_domains) > 0:
+        if len(domain_names) == 1:
+            # One wanted domain but no root company found ?
+            # It can be a user who cannot search directly the root company
+            # So, get the domain from user describe and compare to the wanted one
+            user_domain_root_company = get_domain_root_company_from_current_user(sdk)
+            if user_domain_root_company[0] == domain_names[0]:
+                return [user_domain_root_company]
+
+        if raise_if_not_found:
+            msg = f'Bad domain(s) or no sufficient permissions to access the ' \
+                  f'root company of: {", ".join(not_found_domains)}'
+            print_error(msg, raise_exit=True)
+
+    return [(c.domain, c.id) for c in found_root_companies]
+
+
+@app.command()
+def expose(
+    analytic_name: str = typer.Argument(...),
+    domain: str = typer.Option(
+        default=None,
+        help='To expose the analytic on the specified domains (comma separated values) '
+             'if you have the right permissions on these domains.\n\n'
+             'By default, without providing this option, the analytic will be exposed '
+             'on your domain if you have the right permissions on it.',
+    ),
+):
+    """Expose an analytic"""
+    sdk = alteia_sdk()
+
+    # check the analytic name, will raise error if not found
+    find_analytics(sdk, analytic_name=analytic_name)
+
+    # check domains and get root companies, will raise error if at least one not found
+    domains_root_companies = get_root_companies_of_domains(sdk, domain_names=domain)
+
+    # do expose to each domain one by one
+    for domain_name, root_company_id in domains_root_companies:
+        try:
+            with utils.spinner():
+                sdk.analytics.expose(analytic_name, root_companies=root_company_id)
+            print_ok(f'Analytic "{analytic_name}" exposed successfully '
+                     f'on "{domain_name}" domain')
+        except ResponseError as e:
+            print_error(f'Cannot expose the analytic "{analytic_name}" '
+                        f'on "{domain_name}" domain: {e}')
+
+
+@app.command()
+def unexpose(
+    analytic_name: str = typer.Argument(...),
+    domain: str = typer.Option(
+        default=None,
+        help='To unexpose the analytic from the specified domains (comma separated '
+             'values) if you have the right permissions on these domains.\n\n'
+             'By default, without providing this option, the analytic will be '
+             'unexposed from your domain if you have the right permissions on it.',
+    ),
+):
+    """Unexpose an analytic"""
+    sdk = alteia_sdk()
+
+    # check the analytic name, will raise error if not found
+    find_analytics(sdk, analytic_name=analytic_name)
+
+    # check domains and get root companies, will raise error if at least one not found
+    domains_root_companies = get_root_companies_of_domains(sdk, domain_names=domain)
+
+    # do expose to each domain one by one
+    for domain_name, root_company_id in domains_root_companies:
+        try:
+            with utils.spinner():
+                sdk.analytics.unexpose(analytic_name, root_companies=root_company_id)
+            print_ok(f'Analytic "{analytic_name}" unexposed successfully '
+                     f'from "{domain_name}" domain')
+        except ResponseError as e:
+            print_error(f'Cannot unexpose the analytic "{analytic_name}" '
+                        f'from "{domain_name}" domain: {e}')
+
+
+@app.command(name='list-exposed')
+def list_exposed(
+    display_all: bool = typer.Option(
+        False, '--all', case_sensitive=False,
+        help='If set, display all kinds of analytics '
+             '(otherwise only custom analytics are displayed).'
+    ),
+    domain: str = typer.Option(
+        default=None,
+        help='If set, filters exposed analytics on the specified domains (comma '
+             'separated values) if you have the right permissions on these domains.'
+             '\n\nBy default, without providing this option, '
+             'it filters on your domain.',
+    ),
+):
+    """List exposed analytics"""
+    sdk = alteia_sdk()
+
+    domains_root_companies = get_root_companies_of_domains(sdk, domain_names=domain)
+
+    search_filter = {'is_backoffice': {'$ne': True}}
+    if not display_all:
+        search_filter['external'] = {'$eq': True}
+
+    found_analytics: Dict[str, dict] = {}
+    try:
+        for domain, root_company in domains_root_companies:
+            # do not make one request with all root companies, because we
+            # do not want Intersection results
+            with utils.spinner():
+                results = sdk.analytics.search_generator(
+                    filter=search_filter,
+                    exposed_on_root_companies=[root_company],
+                    fields={'include': ['name', 'display_name', 'external']},
+                )
+            for analytic in results:
+                lower_name = analytic.name.lower()  # for future sorting
+                if not found_analytics.get(lower_name):
+                    found_analytics[lower_name] = {
+                        'name': analytic.name,
+                        'external': getattr(analytic, 'external', False),
+                        'display_name': analytic.display_name,
+                        'domains': set(),
+                    }
+                found_analytics[lower_name]['domains'].add(domain)
+    except ResponseError as e:
+        print_error(f'Cannot search for analytics: {e}', raise_exit=True)
+
+    kind = '' if display_all else 'custom '
+
+    if len(found_analytics) > 0:
+        table: Dict[str, list] = {
+            'Analytic display name': [],
+            'Name': [],
+            'Kind': [],
+            'Exposed domains': [],
+        }
+        for _, a in sorted(found_analytics.items()):  # alphabetic sort on keys
+            table['Analytic display name'].append(green_bold(a['display_name']))
+            table['Name'].append(blue_bold(a['name']))
+            table['Kind'].append('Custom' if a['external'] else 'Catalog')
+            table['Exposed domains'].append(', '.join(sorted(a['domains'])))
+
+        typer.secho(tabulate(
+            table,
+            headers='keys',
+            tablefmt='pretty',
+            colalign='left'
+        ))
+
+        print()
+        print(f'{len(found_analytics)} {kind}analytics displayed')
+
+    else:
+        print(f'No {kind}exposed analytic found.')
+
+
+@app.command()
+def enable(
+    analytic_name: str = typer.Argument(...),
+    company: str = typer.Option(
+        default=None,
+        help='Identifier of the company to enable the analytic, or list of such '
+             'identifiers (comma separated values).\n\n'
+             'When providing the identifier of the root company of your domain, '
+             'the analytic is enabled by default for all the companies of the '
+             'domain (equivalent to using the --domain option).',
+    ),
+    domain: str = typer.Option(
+        default=None,
+        help='Use this option to make the analytic enabled by default for all '
+             'companies of the specified domains (comma separated values) '
+             '(equivalent to using the --company option providing the '
+             'root company identifier(s) of these domains).\n\n'
+             'Apart from this default behavior on domain, the analytic can be '
+             'enabled or disabled separately on each company of the domain.',
+    ),
+):
+    """Enable an analytic on companies"""
+    if company and domain:
+        print_error('Dot not use both options --company and --domain',
+                    raise_exit=True)
+
+    sdk = alteia_sdk()
+
+    # check the analytic name, will raise error if not found
+    find_analytics(sdk, analytic_name=analytic_name)
+
+    if company:
+        # enable directly from company IDs
+        for company in company.split(','):
+            try:
+                sdk.analytics.enable(analytic_name, company)
+                print_ok(f'Analytic "{analytic_name}" successfully enabled '
+                         f'on the company "{company}".')
+            except ResponseError as e:
+                print_error(f'Cannot enable the analytic "{analytic_name}" '
+                            f'on the company "{company}": {e}')
+    elif domain:
+        # check domains and get root companies to enable on them,
+        # will raise error if at least one not found
+        domains_root_companies = get_root_companies_of_domains(sdk, domain_names=domain)
+        for domain_name, root_company_id in domains_root_companies:
+            try:
+                with utils.spinner():
+                    sdk.analytics.enable(analytic_name, root_company_id)
+                print_ok(f'Analytic "{analytic_name}" successfully enabled '
+                         f'by default on domain "{domain_name}" '
+                         f'(root company: {root_company_id})')
+            except ResponseError as e:
+                print_error(f'Cannot enable the analytic "{analytic_name}" '
+                            f'on domain "{domain_name}" '
+                            f'(root company: {root_company_id}): {e}')
+    else:
+        print_error('At least one option --company or --domain must be used',
+                    raise_exit=True)
+
+
+@app.command()
+def disable(
+    analytic_name: str = typer.Argument(...),
+    company: str = typer.Option(
+        default=None,
+        help='Identifier of the company to disable the analytic, or list of such '
+             'identifiers (comma separated values).\n\n'
+             'When providing the identifier of the root company of your domain, '
+             'the analytic is disabled by default for all the companies of the '
+             'domain (equivalent to using the --domain option).',
+    ),
+    domain: str = typer.Option(
+        default=None,
+        help='Use this option to make the analytic disabled by default for all '
+             'companies of the specified domains (comma separated values) '
+             '(equivalent to using the --company option providing the '
+             'root company identifier(s) of these domains).\n\n'
+             'Apart from this default behavior on domain, the analytic can be '
+             'enabled or disabled separately on each company of the domain.',
+    ),
+):
+    """Disable an analytic on companies"""
+    if company and domain:
+        print_error('Dot not use both options --company and --domain',
+                    raise_exit=True)
+
+    sdk = alteia_sdk()
+
+    # check the analytic name, will raise error if not found
+    find_analytics(sdk, analytic_name=analytic_name)
+
+    if company:
+        # disable directly from company IDs
+        for company in company.split(','):
+            try:
+                sdk.analytics.disable(analytic_name, company)
+                print_ok(f'Analytic "{analytic_name}" successfully disabled '
+                         f'on the company "{company}".')
+            except ResponseError as e:
+                print_error(f'Cannot disable the analytic "{analytic_name}" '
+                            f'on the company "{company}": {e}')
+    elif domain:
+        # check domains and get root companies to enable on them,
+        # will raise error if at least one not found
+        domains_root_companies = get_root_companies_of_domains(sdk, domain_names=domain)
+        for domain_name, root_company_id in domains_root_companies:
+            try:
+                with utils.spinner():
+                    sdk.analytics.disable(analytic_name, root_company_id)
+                print_ok(f'Analytic "{analytic_name}" successfully disabled '
+                         f'by default on domain "{domain_name}" '
+                         f'(root company: {root_company_id})')
+            except ResponseError as e:
+                print_error(f'Cannot disable the analytic "{analytic_name}" '
+                            f'on domain "{domain_name}" '
+                            f'(root company: {root_company_id}): {e}')
+    else:
+        print_error('At least one option --company or --domain must be used',
+                    raise_exit=True)
+
+
+@app.command(name='list-orderable')
+def list_orderable(
+    company_id: str = typer.Argument(...),
+    display_all: bool = typer.Option(
+        False, '--all', case_sensitive=False,
+        help='If set, display all kinds of analytics '
+             '(otherwise only custom analytics are displayed).'
+    ),
+):
+    """List orderable analytics on a company"""
+    sdk = alteia_sdk()
+
+    search_filter = {'is_backoffice': {'$ne': True}}
+    if not display_all:
+        search_filter['external'] = {'$eq': True}
+
+    found_analytics: Dict[str, dict] = {}
+    try:
+        with utils.spinner():
+            results = sdk.analytics.search_generator(
+                filter=search_filter,
+                can_be_ordered_by_companies=[company_id],
+                fields={'include': ['name', 'display_name', 'external']},
+                sort={'name': 1},
+            )
+        for analytic in results:
+            # group because of versions
+            if not found_analytics.get(analytic.name):
+                found_analytics[analytic.name] = {
+                    'external': getattr(analytic, 'external', False),
+                    'display_name': analytic.display_name,
+                }
+    except ResponseError as e:
+        print_error(f'Cannot search for analytics: {e}', raise_exit=True)
+
+    kind = '' if display_all else 'custom '
+
+    if len(found_analytics) > 0:
+        table: Dict[str, list] = {
+            'Analytic display name': [],
+            'Name': [],
+            'Kind': [],
+        }
+        for a_name, a in found_analytics.items():
+            table['Analytic display name'].append(green_bold(a['display_name']))
+            table['Name'].append(blue_bold(a_name))
+            table['Kind'].append('Custom' if a['external'] else 'Catalog')
+
+        typer.secho(tabulate(
+            table,
+            headers='keys',
+            tablefmt='pretty',
+            colalign=('left', 'left', 'left'),
+        ))
+
+        print()
+        print(f'{len(found_analytics)} {kind}analytics displayed')
+    else:
+        print(f'No {kind}orderable analytic found.')
