@@ -119,6 +119,12 @@ def print_ok(msg: str, check=True):
     typer.secho(msg, fg=typer.colors.GREEN)
 
 
+def print_warn(msg: str, check=True):
+    if check:
+        msg = f'⚠ {msg}'
+    typer.secho(msg, fg=typer.colors.YELLOW)
+
+
 def print_error(msg: str, check=True, raise_exit=False):
     if check:
         msg = f'✖ {msg}'
